@@ -54,10 +54,14 @@ func NewConfig() Config {
 			Underline: true,
 		},
 		General: general{
-			TabWidth:             8,
-			MarkStyleWidth:       1,
-			SectionStartPosition: 0,
-			JumpTarget:           0,
+			TabWidth:       8,
+			MarkStyleWidth: 1,
+		},
+		Prompt: OVPromptConfig{
+			Normal: OVPromptConfigNormal{
+				ShowFilename: true,
+				InvertColor:  true,
+			},
 		},
 	}
 }
