@@ -1,6 +1,8 @@
 package oviewer
 
 import (
+	"bytes"
+	"errors"
 	"fmt"
 	"io"
 	"io/fs"
@@ -49,8 +51,21 @@ type Document struct {
 	// status is the display status of the document.
 	general
 
+	// memoryLimit is the maximum chunk size.
+	memoryLimit int
+
 	// currentChunk represents the current chunk number.
 	currentChunk int
+
+	// headerLen is the actual header length when wrapped.
+	headerLen int
+	// statusPos is the position of the status line.
+	statusPos int
+
+	// width is the width of the screen.
+	width int
+	// height is the height of the screen.
+	height int
 	// markedPoint is the position of the marked line.
 	markedPoint int
 
@@ -72,6 +87,13 @@ type Document struct {
 	// columnCursor is the number of columns.
 	columnCursor int
 
+	// sectionHeaderNum is the number of lines in the section header.
+	sectionHeaderNum int
+	// jumpTargetNum is the display position of search results.
+	jumpTargetNum int
+	// jumpTargetSection is the display position of search results.
+	jumpTargetSection bool
+
 	// CFormat is a compressed format.
 	CFormat Compressed
 
@@ -82,6 +104,8 @@ type Document struct {
 
 	// 1 if there is a tmpFollow mode.
 	tmpFollow int32
+	// tmpLN is a temporary line number when the number of lines is undetermined.
+	tmpLN int32
 
 	// WatchMode is watch mode.
 	WatchMode bool
@@ -100,7 +124,8 @@ type store struct {
 	// chunks is the content of the file to be stored in chunks.
 	chunks []*chunk
 	// mu controls the mutex.
-	mu sync.Mutex
+	mu sync.RWMutex
+
 	// startNum is the number of the first line that can be moved.
 	startNum int32
 	// endNum is the number of the last line read.
@@ -147,13 +172,12 @@ func NewDocument() (*Document, error) {
 			ColumnDelimiter: "",
 			TabWidth:        8,
 			MarkStyleWidth:  1,
-			PlainMode:       false,
 		},
-		ctlCh:         make(chan controlSpecifier),
-		seekable:      true,
-		reopenable:    true,
-		preventReload: false,
-		store:         NewStore(),
+		ctlCh:       make(chan controlSpecifier),
+		memoryLimit: 100,
+		seekable:    true,
+		reopenable:  true,
+		store:       NewStore(),
 	}
 	if err := m.NewCache(); err != nil {
 		return nil, err
@@ -266,7 +290,7 @@ func (s *store) GetChunkLine(chunkNum int, cn int) ([]byte, error) {
 	if cn >= len(chunk.lines) {
 		return nil, fmt.Errorf("over line (%d:%d) %w", chunkNum, cn, ErrOutOfRange)
 	}
-	return chunk.lines[cn], nil
+	return bytes.TrimSuffix(chunk.lines[cn], []byte("\n")), nil
 }
 
 // GetLine returns one line from buffer.
@@ -281,11 +305,17 @@ func (m *Document) GetLine(n int) string {
 
 // LineString returns one line from buffer.
 func (m *Document) LineString(n int) string {
+	str, _ := m.LineStr(n)
+	return str
+}
+
+// LineStr returns one line from buffer.
+func (m *Document) LineStr(n int) (string, error) {
 	s, err := m.Line(n)
 	if err != nil {
-		return gchalk.Red(err.Error())
+		return gchalk.Red(err.Error()), err
 	}
-	return string(s)
+	return string(s), nil
 }
 
 // CurrentLN returns the currently displayed line number.
@@ -294,13 +324,24 @@ func (m *Document) CurrentLN() int {
 }
 
 // Export exports the document in the specified range.
-func (m *Document) Export(w io.Writer, start int, end int) {
-	for n := start; n <= end; n++ {
-		if n >= m.BufEndNum() {
-			break
+func (m *Document) Export(w io.Writer, start int, end int) error {
+	end = min(end, m.BufEndNum()-1)
+	startChunk, startCn := chunkLineNum(start)
+	endChunk, endCn := chunkLineNum(end)
+
+	scn := startCn
+	ecn := ChunkSize
+	for chunkNum := startChunk; chunkNum <= endChunk; chunkNum++ {
+		if chunkNum == endChunk {
+			ecn = endCn + 1
 		}
-		fmt.Fprint(w, m.LineString(n))
+		chunk := m.store.chunks[chunkNum]
+		if err := m.store.export(w, chunk, scn, ecn); err != nil {
+			return err
+		}
+		scn = 0
 	}
+	return nil
 }
 
 // BufStartNum return start line number.
@@ -337,8 +378,8 @@ func (m *Document) contents(lN int, tabWidth int) (contents, error) {
 		return nil, ErrOutOfRange
 	}
 
-	str := m.LineString(lN)
-	return parseString(str, tabWidth), nil
+	str, err := m.LineStr(lN)
+	return parseString(str, tabWidth), err
 }
 
 // getLineC returns contents from line number and tabWidth.
@@ -352,8 +393,7 @@ func (m *Document) getLineC(lN int, tabWidth int) (LineC, bool) {
 	}
 
 	org, err := m.contents(lN, tabWidth)
-	if err != nil {
-		// EOF
+	if err != nil && errors.Is(err, ErrOutOfRange) {
 		lc := make(contents, 1)
 		lc[0] = EOFContent
 		return LineC{
@@ -368,7 +408,7 @@ func (m *Document) getLineC(lN int, tabWidth int) (LineC, bool) {
 		str: str,
 		pos: pos,
 	}
-	if len(org) != 0 {
+	if err == nil {
 		m.cache.Add(lN, line)
 	}
 
@@ -403,7 +443,7 @@ func (m *Document) unWatchMode() {
 func (m *Document) regexpCompile() {
 	m.ColumnDelimiterReg = condRegexpCompile(m.ColumnDelimiter)
 	m.setSectionDelimiter(m.SectionDelimiter)
-	if m.MultiColorWords != nil {
+	if len(m.MultiColorWords) > 0 {
 		m.setMultiColorWords(m.MultiColorWords)
 	}
 }
@@ -418,6 +458,10 @@ func (m *Document) setDelimiter(delm string) {
 func (m *Document) setSectionDelimiter(delm string) {
 	m.SectionDelimiter = delm
 	m.SectionDelimiterReg = regexpCompile(delm, true)
+	m.sectionHeaderNum = 0
+	if m.SectionHeader && m.SectionDelimiter != "" {
+		m.sectionHeaderNum = 1
+	}
 }
 
 // setMultiColorWords set multiple strings to highlight with multiple colors.
