@@ -22,7 +22,7 @@ func (root *Root) draw() {
 		return
 	}
 
-	if m.ColumnWidth && m.columnWidths == nil {
+	if m.ColumnWidth && len(m.columnWidths) == 0 {
 		m.setColumnWidths()
 	}
 
@@ -35,6 +35,14 @@ func (root *Root) draw() {
 	}
 
 	lN = m.topLN + lN
+
+	// Section header
+	n := root.drawSectionHeader(lN)
+	if lN != n {
+		lN = n
+		m.topLN = lN
+	}
+
 	// Body
 	lX, lN = root.drawBody(lX, lN)
 
@@ -80,16 +88,15 @@ func (root *Root) drawHeader() int {
 			root.blankLineNumber(y)
 		}
 
-		nextX, nextY := root.drawLine(y, lX, lN, line.lc)
+		nextX, nextN := root.drawLine(y, lX, lN, line.lc)
 		// header style
 		root.yStyle(y, root.StyleHeader)
 
 		lX = nextX
-		if nextY != lN {
-			lN = nextY
+		if nextN != lN {
+			lN = nextN
 			line, _ = m.getLineC(lN, m.TabWidth)
 			root.styleContent(lN, line)
-
 		}
 
 		if lX > 0 {
@@ -98,21 +105,66 @@ func (root *Root) drawHeader() int {
 			wrapNum = 0
 		}
 	}
-	root.headerLen = y
+	m.headerLen = y
 	return lN
 }
 
+// drawSectionHeader draws section header.
+func (root *Root) drawSectionHeader(lN int) int {
+	m := root.Doc
+	if !m.SectionHeader || m.SectionDelimiter == "" {
+		return lN
+	}
+
+	pn := lN
+	if pn == 0 {
+		pn = 1
+	}
+	sectionLN, err := m.prevSection(pn)
+	if err != nil {
+		return lN
+	}
+
+	if m.Header > sectionLN {
+		return pn
+	}
+
+	sx, sn := 0, 0
+	nextN := 0
+	if pn > sectionLN {
+		sn = sectionLN
+		line, valid := m.getLineC(sn, m.TabWidth)
+		for y := m.headerLen; sn < sectionLN+m.sectionHeaderNum; y++ {
+			if root.Doc.LineNumMode {
+				if valid {
+					root.drawLineNumber(sn, y)
+				} else {
+					root.blankLineNumber(y)
+				}
+			}
+			sx, nextN = root.drawLine(y, sx, sn, line.lc)
+			root.sectionLineHighlight(y, line.str)
+			m.headerLen += 1
+			if nextN != sn {
+				sn = nextN
+				line, valid = m.getLineC(sn, m.TabWidth)
+			}
+		}
+	}
+	return pn
+}
+
 // drawBody draws body.
 func (root *Root) drawBody(lX int, lN int) (int, int) {
 	m := root.Doc
 
-	wrapNum := root.numOfWrap(lX, lN)
+	wrapNum := m.numOfWrap(lX, lN)
 	line, valid := m.getLineC(lN, m.TabWidth)
 	root.bodyStyle(line.lc, root.StyleBody)
 	if valid {
 		root.styleContent(lN, line)
 	}
-	for y := root.headerLen; y < root.scr.vHeight-statusLine; y++ {
+	for y := m.headerLen; y < root.scr.vHeight-statusLine; y++ {
 		root.scr.numbers[y] = LineNumber{
 			number: lN,
 			wrap:   wrapNum,
@@ -151,6 +203,7 @@ func (root *Root) drawBody(lX int, lN int) (int, int) {
 	return lX, lN
 }
 
+// styleContent applies the style of the content.
 func (root *Root) styleContent(lN int, line LineC) {
 	if root.Doc.PlainMode {
 		root.plainStyle(line.lc)
@@ -162,12 +215,13 @@ func (root *Root) styleContent(lN int, line LineC) {
 	root.searchHighlight(lN, line)
 }
 
+// coordinatesStyle applies the style of the coordinates.
 func (root *Root) coordinatesStyle(lN int, y int, str string) {
 	root.alternateRowsStyle(lN, y)
 	markStyleWidth := min(root.scr.vWidth, root.Doc.general.MarkStyleWidth)
 	root.markStyle(lN, y, markStyleWidth)
 	root.sectionLineHighlight(y, str)
-	if root.Doc.JumpTarget != 0 && root.headerLen+root.Doc.JumpTarget == y {
+	if root.Doc.jumpTargetNum != 0 && root.Doc.headerLen+root.Doc.jumpTargetNum == y {
 		root.yStyle(y, root.StyleJumpTargetLine)
 	}
 }
@@ -210,26 +264,23 @@ func (root *Root) drawWrapLine(y int, lX int, lN int, lc contents) (int, int) {
 }
 
 // drawNoWrapLine draws contents without wrapping and returns the next drawing position.
-func (root *Root) drawNoWrapLine(y int, lX int, lN int, lc contents) (int, int) {
-	if lX < root.minStartX {
-		lX = root.minStartX
-	}
-
+func (root *Root) drawNoWrapLine(y int, startX int, lN int, lc contents) (int, int) {
+	startX = max(startX, root.minStartX)
 	for x := 0; root.scr.startX+x < root.scr.vWidth; x++ {
-		if lX+x >= len(lc) {
+		if startX+x >= len(lc) {
 			// EOL
 			root.clearEOL(root.scr.startX+x, y)
 			break
 		}
 		content := DefaultContent
-		if lX+x >= 0 {
-			content = lc[lX+x]
+		if startX+x >= 0 {
+			content = lc[startX+x]
 		}
 		root.Screen.SetContent(root.scr.startX+x, y, content.mainc, content.combc, content.style)
 	}
 	lN++
 
-	return lX, lN
+	return startX, lN
 }
 
 // bodyStyle applies the style from the beginning to the end of one line of the body.
@@ -241,7 +292,7 @@ func (root *Root) bodyStyle(lc contents, s OVStyle) {
 // searchHighlight applies the style of the search highlight.
 // Apply style to contents.
 func (root *Root) searchHighlight(lN int, line LineC) {
-	if root.searchWord == "" {
+	if root.searcher == nil || root.searcher.String() == "" {
 		return
 	}
 
@@ -304,13 +355,20 @@ func (root *Root) columnDelimiterHighlight(line LineC) {
 	if len(indexes) == 0 {
 		return
 	}
+
+	lStart := 0
+	if indexes[0][0] == 0 {
+		lStart = indexes[0][1]
+		indexes = indexes[1:]
+	}
+
 	numC := len(root.StyleColumnRainbow)
 
 	var iStart, iEnd int
 	for c := 0; c < len(indexes)+1; c++ {
 		switch {
 		case c == 0:
-			iStart = 0
+			iStart = lStart
 			iEnd = indexes[0][1] - 1
 			if iEnd < 0 {
 				iEnd = 0
@@ -443,16 +501,17 @@ func (root *Root) markStyle(lN int, y int, width int) {
 
 // drawStatus draws a status line.
 func (root *Root) drawStatus() {
-	root.clearY(root.statusPos)
+	root.clearY(root.Doc.statusPos)
 	leftContents, cursorPos := root.leftStatus()
-	root.setContentString(0, root.statusPos, leftContents)
+	root.setContentString(0, root.Doc.statusPos, leftContents)
 
 	rightContents := root.rightStatus()
-	root.setContentString(root.scr.vWidth-len(rightContents), root.statusPos, rightContents)
+	root.setContentString(root.scr.vWidth-len(rightContents), root.Doc.statusPos, rightContents)
 
-	root.Screen.ShowCursor(cursorPos, root.statusPos)
+	root.Screen.ShowCursor(cursorPos, root.Doc.statusPos)
 }
 
+// leftStatus returns the status of the left side.
 func (root *Root) leftStatus() (contents, int) {
 	if root.input.Event.Mode() == Normal {
 		return root.normalLeftStatus()
@@ -460,6 +519,7 @@ func (root *Root) leftStatus() (contents, int) {
 	return root.inputLeftStatus()
 }
 
+// normalLeftStatus returns the status of the left side of the normal mode.
 func (root *Root) normalLeftStatus() (contents, int) {
 	number := ""
 	if root.DocumentLen() > 1 && root.screenMode == Docs {
@@ -483,23 +543,31 @@ func (root *Root) normalLeftStatus() (contents, int) {
 	} else if root.Doc.FollowSection {
 		modeStatus = "(Follow Section)"
 	}
-	caption := root.Doc.FileName
+
+	caption := ""
 	if root.Doc.Caption != "" {
 		caption = root.Doc.Caption
+	} else if root.Config.Prompt.Normal.ShowFilename {
+		caption = root.Doc.FileName
 	}
 
 	leftStatus := fmt.Sprintf("%s%s%s:%s", number, modeStatus, caption, root.message)
 	leftContents := StrToContents(leftStatus, -1)
-	color := tcell.ColorWhite
-	if root.CurrentDoc != 0 {
-		color = tcell.Color((root.CurrentDoc + 8) % 16)
-	}
-	for i := 0; i < len(leftContents); i++ {
-		leftContents[i].style = leftContents[i].style.Foreground(tcell.ColorValid + color).Reverse(true)
+
+	if root.Config.Prompt.Normal.InvertColor {
+		color := tcell.ColorWhite
+		if root.CurrentDoc != 0 {
+			color = tcell.Color((root.CurrentDoc + 8) % 16)
+		}
+		for i := 0; i < len(leftContents); i++ {
+			leftContents[i].style = leftContents[i].style.Foreground(tcell.ColorValid + color).Reverse(true)
+		}
 	}
+
 	return leftContents, len(leftContents)
 }
 
+// inputLeftStatus returns the status of the left side of the input.
 func (root *Root) inputLeftStatus() (contents, int) {
 	input := root.input
 	p := root.inputOpts() + input.Event.Prompt()
@@ -521,7 +589,9 @@ func (root *Root) inputOpts() string {
 		if root.Config.Incsearch {
 			opts += "(I)"
 		}
-		if root.Config.CaseSensitive {
+		if root.Config.SmartCaseSensitive {
+			opts += "(S)"
+		} else if root.Config.CaseSensitive {
 			opts += "(Aa)"
 		}
 	}
@@ -529,6 +599,7 @@ func (root *Root) inputOpts() string {
 	return opts
 }
 
+// rightStatus returns the status of the right side.
 func (root *Root) rightStatus() contents {
 	next := ""
 	if !root.Doc.BufEOF() {
