@@ -5,57 +5,46 @@ import (
 )
 
 func TestDocument_Write(t *testing.T) {
+	t.Parallel()
 	type args struct {
-		logs [][]byte
+		log []byte
 	}
 	tests := []struct {
-		name      string
-		ChunkSize int
-		args      args
-		want      int
-		wantErr   bool
+		name    string
+		repeat  int
+		args    args
+		want    int
+		wantErr bool
 	}{
 		{
-			name:      "test1",
-			ChunkSize: 10000,
+			name:   "test1",
+			repeat: 1,
 			args: args{
-				logs: [][]byte{
-					[]byte("test"),
-				},
+				log: []byte("test"),
 			},
 			want:    4,
 			wantErr: false,
 		},
 		{
-			name:      "testChunkSize",
-			ChunkSize: 2,
+			name:   "testChunkSize",
+			repeat: 30002,
 			args: args{
-				logs: [][]byte{
-					[]byte("test1"),
-					[]byte("test2"),
-					[]byte("test3"),
-					[]byte("test4"),
-					[]byte("test5"),
-					[]byte("test6"),
-				},
+				log: []byte("test"),
 			},
-			want:    5,
+			want:    4,
 			wantErr: false,
 		},
 	}
 	for _, tt := range tests {
+		tt := tt
 		t.Run(tt.name, func(t *testing.T) {
-			tmp := ChunkSize
-			ChunkSize = tt.ChunkSize
-			defer func() {
-				ChunkSize = tmp
-			}()
+			t.Parallel()
 			m, err := NewDocument()
 			if err != nil {
 				t.Fatal(err)
 			}
-			for _, log := range tt.args.logs {
-				got, err := m.Write(log)
+			for i := 0; i < tt.repeat; i++ {
+				got, err := m.Write(tt.args.log)
 				if (err != nil) != tt.wantErr {
 					t.Errorf("Document.Write() error = %v, wantErr %v", err, tt.wantErr)
 					return
