@@ -34,15 +34,14 @@ type Root struct {
 	// cancelFunc saves the cancel function, which is a time-consuming process.
 	cancelFunc context.CancelFunc
 
-	// searchReg for on-screen highlighting.
-	searchReg *regexp.Regexp
+	// searcher is the searcher.
+	searcher Searcher
+
 	// keyConfig contains the binding settings for the key.
 	keyConfig *cbind.Configuration
 	// inputKeyConfig contains the binding settings for the key.
 	inputKeyConfig *cbind.Configuration
 
-	// searchWord for on-screen highlighting.
-	searchWord string
 	// Original string.
 	OriginStr string
 
@@ -70,11 +69,6 @@ type Root struct {
 	x2 int
 	y2 int
 
-	// headerLen is the actual header length when wrapped.
-	headerLen int
-	// statusPos is the position of the status line.
-	statusPos int
-
 	// mu controls the RWMutex.
 	mu sync.RWMutex
 
@@ -119,7 +113,7 @@ type general struct {
 	// SectionDelimiter is a section delimiter.
 	SectionDelimiter string
 	// Specified string for jumpTarget.
-	JumpTargetString string
+	JumpTarget string
 	// MultiColorWords specifies words to color separated by spaces.
 	MultiColorWords []string
 
@@ -135,8 +129,6 @@ type general struct {
 	MarkStyleWidth int
 	// SectionStartPosition is a section start position.
 	SectionStartPosition int
-	// JumpTarget is the display position of search results.
-	JumpTarget int
 	// AlternateRows alternately style rows.
 	AlternateRows bool
 	// ColumnMode is column mode.
@@ -159,39 +151,96 @@ type general struct {
 	FollowName bool
 	// PlainMode is whether to enable the original character decoration.
 	PlainMode bool
+	// SectionHeader is whether to display the section header.
+	SectionHeader bool
+}
+
+// OVPromptConfigNormal is the normal prompt setting.
+type OVPromptConfigNormal struct {
+	// ShowFilename controls whether to display filename.
+	ShowFilename bool
+	// InvertColor controls whether the text is colored and inverted.
+	InvertColor bool
+}
+
+// OVPromptConfig is the prompt setting.
+type OVPromptConfig struct {
+	// Normal is the normal prompt setting.
+	Normal OVPromptConfigNormal
 }
 
 // Config represents the settings of ov.
 type Config struct {
-	Keybind                  map[string][]string
-	Mode                     map[string]general
-	ViewMode                 string
-	DefaultKeyBind           string
-	StyleColumnRainbow       []OVStyle
+	// KeyBinding
+	Keybind map[string][]string
+	// Mode represents the operation of the customized mode.
+	Mode map[string]general
+	// ViewMode represents the view mode.
+	// ViewMode sets several settings together and can be easily switched.
+	ViewMode string
+	// Default keybindings. Disabled if the default keybinding is "disable".
+	DefaultKeyBind string
+	// StyleColumnRainbow  is the style that applies to the column rainbow color highlight.
+	StyleColumnRainbow []OVStyle
+	// StyleMultiColorHighlight is the style that applies to the multi color highlight.
 	StyleMultiColorHighlight []OVStyle
-	StyleLineNumber          OVStyle
-	StyleHeader              OVStyle
-	StyleSearchHighlight     OVStyle
-	StyleColumnHighlight     OVStyle
-	StyleMarkLine            OVStyle
-	StyleSectionLine         OVStyle
-	StyleOverStrike          OVStyle
-	StyleBody                OVStyle
-	StyleJumpTargetLine      OVStyle
-	StyleAlternate           OVStyle
-	StyleOverLine            OVStyle
-	General                  general
-	MemoryLimitFile          int
-	AfterWriteOriginal       int
-	BeforeWriteOriginal      int
-	MemoryLimit              int
-	DisableMouse             bool
-	IsWriteOriginal          bool
-	QuitSmall                bool
-	CaseSensitive            bool
-	RegexpSearch             bool
-	Incsearch                bool
-	Debug                    bool
+
+	// Prompt is the prompt setting.
+	Prompt OVPromptConfig
+
+	// StyleHeader is the style that applies to the header.
+	StyleHeader OVStyle
+	// StyleBody is the style that applies to the body.
+	StyleBody OVStyle
+	// StyleLineNumber is a style that applies line number.
+	StyleLineNumber OVStyle
+	// StyleSearchHighlight is the style that applies to the search highlight.
+	StyleSearchHighlight OVStyle
+	// StyleColumnHighlight is the style that applies to the column highlight.
+	StyleColumnHighlight OVStyle
+	// StyleMarkLine is a style that marked line.
+	StyleMarkLine OVStyle
+	// StyleSectionLine is a style that section delimiter line.
+	StyleSectionLine OVStyle
+	// StyleJumpTargetLine is the line that displays the search results.
+	StyleJumpTargetLine OVStyle
+	// StyleAlternate is a style that applies line by line.
+	StyleAlternate OVStyle
+	// StyleOverStrike is a style that applies to overstrike.
+	StyleOverStrike OVStyle
+	// StyleOverLine is a style that applies to overstrike underlines.
+	StyleOverLine OVStyle
+	// General represents the general behavior.
+	General general
+	// BeforeWriteOriginal specifies the number of lines before the current position.
+	// 0 is the top of the current screen
+	BeforeWriteOriginal int
+	// AfterWriteOriginal specifies the number of lines after the current position.
+	// 0 specifies the bottom of the screen.
+	AfterWriteOriginal int
+	// MemoryLimit is a number that limits chunk loading.
+	MemoryLimit int
+	// MemoryLimitFile is a number that limits the chunks loading a file into memory.
+	MemoryLimitFile int
+	// Mouse support disable.
+	DisableMouse bool
+	// IsWriteOriginal is true, write the current screen on quit.
+	IsWriteOriginal bool
+	// QuitSmall Quit if the output fits on one screen.
+	QuitSmall bool
+	// CaseSensitive is case-sensitive if true.
+	CaseSensitive bool
+	// SmartCaseSensitive is lowercase search ignores case, if true.
+	SmartCaseSensitive bool
+	// RegexpSearch is Regular expression search if true.
+	RegexpSearch bool
+	// Incsearch is incremental search if true.
+	Incsearch bool
+
+	// DisableColumnCycle is disable column cycle.
+	DisableColumnCycle bool
+	// Debug represents whether to enable the debug output.
+	Debug bool
 }
 
 // OVStyle represents a style in addition to the original style.
@@ -244,6 +293,8 @@ var (
 	OverStrikeStyle tcell.Style
 	// OverLineStyle represents the overline underline style.
 	OverLineStyle tcell.Style
+	// SkipExtract is a flag to skip extracting compressed files.
+	SkipExtract bool
 )
 
 // ov output destination.
@@ -304,6 +355,8 @@ var (
 	ErrNoColumn = errors.New("no column")
 	// ErrNoDelimiter indicates that the line containing the delimiter could not be found.
 	ErrNoDelimiter = errors.New("no delimiter")
+	// ErrOverScreen indicates that the specified screen is out of range.
+	ErrOverScreen = errors.New("over screen")
 	// ErrOutOfChunk indicates that the specified Chunk is out of range.
 	ErrOutOfChunk = errors.New("out of chunk")
 	// ErrNotLoaded indicates that it cannot be loaded.
@@ -452,32 +505,7 @@ func (root *Root) SetWatcher(watcher *fsnotify.Watcher) {
 				if !ok {
 					return
 				}
-				root.mu.Lock()
-				for _, doc := range root.DocList {
-					if doc.filepath == event.Name {
-						switch event.Op {
-						case fsnotify.Write:
-							select {
-							case doc.ctlCh <- controlSpecifier{request: requestFollow}:
-								root.debugMessage(fmt.Sprintf("notify send %v", event))
-							default:
-								root.debugMessage(fmt.Sprintf("notify send fail %s", requestFollow))
-							}
-						case fsnotify.Remove, fsnotify.Create:
-							if !doc.FollowName {
-								continue
-							}
-							select {
-							case doc.ctlCh <- controlSpecifier{request: requestReload}:
-								root.debugMessage(fmt.Sprintf("notify send %v", event))
-							default:
-								root.debugMessage(fmt.Sprintf("notify send fail %s", requestReload))
-							}
-
-						}
-					}
-				}
-				root.mu.Unlock()
+				root.watchEvent(event)
 			case err, ok := <-watcher.Errors:
 				if !ok {
 					return
@@ -502,6 +530,34 @@ func (root *Root) SetWatcher(watcher *fsnotify.Watcher) {
 	}
 }
 
+// watchEvent sends a notification to the document.
+func (root *Root) watchEvent(event fsnotify.Event) {
+	root.mu.Lock()
+	defer root.mu.Unlock()
+
+	for _, m := range root.DocList {
+		if m.filepath == event.Name {
+			switch event.Op {
+			case fsnotify.Write:
+				root.sendRequest(m, requestFollow)
+			case fsnotify.Remove, fsnotify.Create:
+				if m.FollowName {
+					root.sendRequest(m, requestReload)
+				}
+			}
+		}
+	}
+}
+
+func (root *Root) sendRequest(m *Document, request request) {
+	select {
+	case m.ctlCh <- controlSpecifier{request: request}:
+		root.debugMessage(fmt.Sprintf("notify send %v", request))
+	default:
+		root.debugMessage(fmt.Sprintf("notify send fail %s", request))
+	}
+}
+
 // setKeyConfig sets key bindings.
 func (root *Root) setKeyConfig() (map[string][]string, error) {
 	keyBind := GetKeyBinds(root.Config)
@@ -632,6 +688,12 @@ func (root *Root) Close() {
 	root.Screen.Fini()
 }
 
+// setMessagef displays a formatted message in status.
+func (root *Root) setMessagef(format string, a ...any) {
+	msg := fmt.Sprintf(format, a...)
+	root.setMessage(msg)
+}
+
 // setMessage displays a message in status.
 func (root *Root) setMessage(msg string) {
 	if root.message == msg {
@@ -643,11 +705,24 @@ func (root *Root) setMessage(msg string) {
 	root.Show()
 }
 
-func (root *Root) setMessagef(format string, a ...any) {
+// setMessageLogf displays a formatted message in the status and outputs it to the log.
+func (root *Root) setMessageLogf(format string, a ...any) {
 	msg := fmt.Sprintf(format, a...)
-	root.setMessage(msg)
+	root.setMessageLog(msg)
 }
 
+// setMessageLog displays a message in the status and outputs it to the log.
+func (root *Root) setMessageLog(msg string) {
+	if root.message == msg {
+		return
+	}
+	root.message = msg
+	log.Print(msg)
+	root.drawStatus()
+	root.Show()
+}
+
+// debugMessage outputs a debug message.
 func (root *Root) debugMessage(msg string) {
 	if !root.Debug {
 		return
@@ -655,7 +730,7 @@ func (root *Root) debugMessage(msg string) {
 	if root.Doc == root.logDoc {
 		return
 	}
-	root.message = msg
+
 	if len(msg) == 0 {
 		return
 	}
@@ -786,13 +861,10 @@ func mergeGeneral(src general, dst general) general {
 	if dst.SectionStartPosition != 0 {
 		src.SectionStartPosition = dst.SectionStartPosition
 	}
-	if dst.JumpTargetString != "" {
-		src.JumpTargetString = dst.JumpTargetString
-	}
-	if dst.JumpTarget != 0 {
+	if dst.JumpTarget != "" {
 		src.JumpTarget = dst.JumpTarget
 	}
-	if dst.MultiColorWords != nil {
+	if len(dst.MultiColorWords) > 0 {
 		src.MultiColorWords = dst.MultiColorWords
 	}
 	return src
@@ -807,7 +879,7 @@ func (root *Root) prepareView() {
 	root.scr.vWidth = max(root.scr.vWidth, 1)
 	root.scr.vHeight = max(root.scr.vHeight, 1)
 	root.scr.numbers = make([]LineNumber, root.scr.vHeight+1)
-	root.statusPos = root.scr.vHeight - statusLine
+	root.Doc.statusPos = root.scr.vHeight - statusLine
 }
 
 // docSmall returns with bool whether the file to display fits on the screen.
@@ -848,7 +920,9 @@ func (root *Root) WriteOriginal() {
 		end = m.topLN + root.AfterWriteOriginal - 1
 	}
 
-	m.Export(os.Stdout, start, end)
+	if err := m.Export(os.Stdout, start, end); err != nil {
+		log.Println(err)
+	}
 }
 
 // WriteLog write to the log terminal.
@@ -856,7 +930,9 @@ func (root *Root) WriteLog() {
 	m := root.logDoc
 	start := max(0, m.BufEndNum()-MaxWriteLog)
 	end := m.BufEndNum()
-	m.Export(os.Stdout, start, end)
+	if err := m.Export(os.Stdout, start, end); err != nil {
+		log.Println(err)
+	}
 }
 
 // lineNumber returns the line information from y on the screen.
@@ -867,6 +943,7 @@ func (scr SCR) lineNumber(y int) LineNumber {
 	return scr.numbers[0]
 }
 
+// debugNumOfChunk outputs the number of chunks.
 func (root *Root) debugNumOfChunk() {
 	if !root.Debug {
 		return
